@@ -136,9 +136,16 @@ static HRESULT STDMETHODCALLTYPE d3d11_query_SetPrivateDataInterface(ID3D11Query
 
 static UINT STDMETHODCALLTYPE d3d11_query_GetDataSize(ID3D11Query *iface)
 {
-    FIXME("iface %p stub!\n", iface);
+    struct d3d_query *query = impl_from_ID3D11Query(iface);
+    unsigned int data_size;
 
-    return 0;
+    TRACE("iface %p.\n", iface);
+
+    wined3d_mutex_lock();
+    data_size = wined3d_query_get_data_size(query->wined3d_query);
+    wined3d_mutex_unlock();
+
+    return data_size;
 }
 
 static void STDMETHODCALLTYPE d3d11_query_GetDesc(ID3D11Query *iface, D3D11_QUERY_DESC *desc)
@@ -171,6 +178,11 @@ struct d3d_query *unsafe_impl_from_ID3D11Query(ID3D11Query *iface)
     return CONTAINING_RECORD(iface, struct d3d_query, ID3D11Query_iface);
 }
 
+struct d3d_query *unsafe_impl_from_ID3D11Asynchronous(ID3D11Asynchronous *iface)
+{
+    return unsafe_impl_from_ID3D11Query((ID3D11Query *)iface);
+}
+
 /* ID3D10Query methods */
 
 static inline struct d3d_query *impl_from_ID3D10Query(ID3D10Query *iface)
@@ -254,26 +266,67 @@ static HRESULT STDMETHODCALLTYPE d3d10_query_SetPrivateDataInterface(ID3D10Query
 
 static void STDMETHODCALLTYPE d3d10_query_Begin(ID3D10Query *iface)
 {
-    FIXME("iface %p stub!\n", iface);
+    struct d3d_query *query = impl_from_ID3D10Query(iface);
+    HRESULT hr;
+
+    TRACE("iface %p.\n", iface);
+
+    wined3d_mutex_lock();
+    if (FAILED(hr = wined3d_query_issue(query->wined3d_query, WINED3DISSUE_BEGIN)))
+        ERR("Failed to issue query, hr %#x.\n", hr);
+    wined3d_mutex_unlock();
 }
 
 static void STDMETHODCALLTYPE d3d10_query_End(ID3D10Query *iface)
 {
-    FIXME("iface %p stub!\n", iface);
+    struct d3d_query *query = impl_from_ID3D10Query(iface);
+    HRESULT hr;
+
+    TRACE("iface %p.\n", iface);
+
+    wined3d_mutex_lock();
+    if (FAILED(hr = wined3d_query_issue(query->wined3d_query, WINED3DISSUE_END)))
+        ERR("Failed to issue query, hr %#x.\n", hr);
+    wined3d_mutex_unlock();
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_query_GetData(ID3D10Query *iface, void *data, UINT data_size, UINT flags)
 {
-    FIXME("iface %p, data %p, data_size %u, flags %#x stub!\n", iface, data, data_size, flags);
+    struct d3d_query *query = impl_from_ID3D10Query(iface);
+    unsigned int wined3d_flags;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, data %p, data_size %u, flags %#x.\n", iface, data, data_size, flags);
+
+    wined3d_flags = wined3d_getdata_flags_from_d3d11_async_getdata_flags(flags);
+
+    wined3d_mutex_lock();
+    if (!data_size || wined3d_query_get_data_size(query->wined3d_query) == data_size)
+    {
+        hr = wined3d_query_get_data(query->wined3d_query, data, data_size, wined3d_flags);
+    }
+    else
+    {
+        WARN("Invalid data size %u.\n", data_size);
+        hr = E_INVALIDARG;
+    }
+    wined3d_mutex_unlock();
+
+    return hr;
 }
 
 static UINT STDMETHODCALLTYPE d3d10_query_GetDataSize(ID3D10Query *iface)
 {
-    FIXME("iface %p stub!\n", iface);
+    struct d3d_query *query = impl_from_ID3D10Query(iface);
+    unsigned int data_size;
 
-    return 0;
+    TRACE("iface %p.\n", iface);
+
+    wined3d_mutex_lock();
+    data_size = wined3d_query_get_data_size(query->wined3d_query);
+    wined3d_mutex_unlock();
+
+    return data_size;
 }
 
 /* ID3D10Query methods */
@@ -364,23 +417,28 @@ HRESULT d3d_query_create(struct d3d_device *device, const D3D11_QUERY_DESC *desc
         struct d3d_query **query)
 {
     struct d3d_query *object;
+    BOOL is_predicate_type;
     HRESULT hr;
 
     if (!desc)
         return E_INVALIDARG;
 
-    if (predicate
-            && desc->Query != D3D11_QUERY_OCCLUSION_PREDICATE
-            && desc->Query != D3D11_QUERY_SO_OVERFLOW_PREDICATE
-            && desc->Query != D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM0
-            && desc->Query != D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM1
-            && desc->Query != D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM2
-            && desc->Query != D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM3)
+    is_predicate_type = desc->Query == D3D11_QUERY_OCCLUSION_PREDICATE
+            || desc->Query == D3D11_QUERY_SO_OVERFLOW_PREDICATE
+            || desc->Query == D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM0
+            || desc->Query == D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM1
+            || desc->Query == D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM2
+            || desc->Query == D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM3;
+
+    if (!is_predicate_type && predicate)
     {
         WARN("Query type %u is not a predicate.\n", desc->Query);
         return E_INVALIDARG;
     }
 
+    if (is_predicate_type)
+        predicate = TRUE;
+
     if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
         return E_OUTOFMEMORY;
 
