@@ -335,6 +335,7 @@ static HRESULT RegExp_set_lastIndex(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t
 
     TRACE("\n");
 
+    jsval_release(regexp->last_index_val);
     hres = jsval_copy(value, &regexp->last_index_val);
     if(FAILED(hres))
         return hres;
@@ -373,8 +374,8 @@ static HRESULT RegExp_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     if(f & REG_MULTILINE)
         len++;
 
-    ptr = jsstr_alloc_buf(len, &ret);
-    if(!ptr)
+    ret = jsstr_alloc_buf(len, &ptr);
+    if(!ret)
         return E_OUTOFMEMORY;
 
     *ptr++ = '/';
