@@ -26,8 +26,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 
-#define UINT32_MAX 0xffffffff
-
 typedef struct {
     jsdisp_t dispex;
     jsstr_t *str;
@@ -68,9 +66,14 @@ static const WCHAR toLocaleUpperCaseW[] = {'t','o','L','o','c','a','l','e','U','
 static const WCHAR localeCompareW[] = {'l','o','c','a','l','e','C','o','m','p','a','r','e',0};
 static const WCHAR fromCharCodeW[] = {'f','r','o','m','C','h','a','r','C','o','d','e',0};
 
+static inline StringInstance *string_from_jsdisp(jsdisp_t *jsdisp)
+{
+    return CONTAINING_RECORD(jsdisp, StringInstance, dispex);
+}
+
 static inline StringInstance *string_from_vdisp(vdisp_t *vdisp)
 {
-    return (StringInstance*)vdisp->u.jsdisp;
+    return string_from_jsdisp(vdisp->u.jsdisp);
 }
 
 static inline StringInstance *string_this(vdisp_t *jsthis)
@@ -106,26 +109,22 @@ static HRESULT get_string_flat_val(script_ctx_t *ctx, vdisp_t *jsthis, jsstr_t *
     return E_OUTOFMEMORY;
 }
 
-static HRESULT String_length(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
-        jsval_t *r)
+static HRESULT String_get_length(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
+    StringInstance *string = (StringInstance*)jsthis;
+
     TRACE("%p\n", jsthis);
 
-    switch(flags) {
-    case DISPATCH_PROPERTYGET: {
-        StringInstance *string = string_from_vdisp(jsthis);
-
-        *r = jsval_number(jsstr_length(string->str));
-        break;
-    }
-    default:
-        FIXME("unimplemented flags %x\n", flags);
-        return E_NOTIMPL;
-    }
-
+    *r = jsval_number(jsstr_length(string->str));
     return S_OK;
 }
 
+static HRESULT String_set_length(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
+{
+    FIXME("%p\n", jsthis);
+    return E_NOTIMPL;
+}
+
 static HRESULT stringobj_to_string(vdisp_t *jsthis, jsval_t *r)
 {
     StringInstance *string;
@@ -176,7 +175,7 @@ static HRESULT do_attributeless_tag_format(script_ctx_t *ctx, vdisp_t *jsthis, j
 
     tagname_len = strlenW(tagname);
 
-    ptr = jsstr_alloc_buf(jsstr_length(str) + 2*tagname_len + 5, &ret);
+    ret = jsstr_alloc_buf(jsstr_length(str) + 2*tagname_len + 5, &ptr);
     if(!ret) {
         jsstr_release(str);
         return E_OUTOFMEMORY;
@@ -226,8 +225,8 @@ static HRESULT do_attribute_tag_format(script_ctx_t *ctx, vdisp_t *jsthis, unsig
         jsstr_t *ret;
         WCHAR *ptr;
 
-        ptr = jsstr_alloc_buf(2*tagname_len + attrname_len + jsstr_length(attr_value) + jsstr_length(str) + 9, &ret);
-        if(ptr) {
+        ret = jsstr_alloc_buf(2*tagname_len + attrname_len + jsstr_length(attr_value) + jsstr_length(str) + 9, &ptr);
+        if(ret) {
             *ptr++ = '<';
             memcpy(ptr, tagname, tagname_len*sizeof(WCHAR));
             ptr += tagname_len;
@@ -377,7 +376,7 @@ static HRESULT String_charCodeAt(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
 static HRESULT String_concat(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    jsstr_t *ret, *str;
+    jsstr_t *ret = NULL, *str;
     HRESULT hres;
 
     TRACE("\n");
@@ -434,8 +433,8 @@ static HRESULT String_concat(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
             }
 
             if(SUCCEEDED(hres)) {
-                ptr = jsstr_alloc_buf(len, &ret);
-                if(ptr) {
+                ret = jsstr_alloc_buf(len, &ptr);
+                if(ret) {
                     for(i=0; i < str_cnt; i++)
                         ptr += jsstr_flush(strs[i], ptr);
                 }else {
@@ -487,9 +486,9 @@ static HRESULT String_fontsize(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
 static HRESULT String_indexOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
+    unsigned pos = 0, search_len, length;
     jsstr_t *search_jsstr, *jsstr;
     const WCHAR *search_str, *str;
-    int length, pos = 0;
     INT ret = -1;
     HRESULT hres;
 
@@ -499,7 +498,6 @@ static HRESULT String_indexOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
     if(FAILED(hres))
         return hres;
 
-    length = jsstr_length(jsstr);
     if(!argc) {
         if(r)
             *r = jsval_number(-1);
@@ -513,6 +511,9 @@ static HRESULT String_indexOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
         return hres;
     }
 
+    search_len = jsstr_length(search_jsstr);
+    length = jsstr_length(jsstr);
+
     if(argc >= 2) {
         double d;
 
@@ -521,14 +522,16 @@ static HRESULT String_indexOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
             pos = is_int32(d) ? min(length, d) : length;
     }
 
-    if(SUCCEEDED(hres)) {
+    if(SUCCEEDED(hres) && length >= search_len) {
+        const WCHAR *end = str+length-search_len;
         const WCHAR *ptr;
 
-        ptr = strstrW(str+pos, search_str);
-        if(ptr)
-            ret = ptr - str;
-        else
-            ret = -1;
+        for(ptr = str+pos; ptr <= end; ptr++) {
+            if(!memcmp(ptr, search_str, search_len*sizeof(WCHAR))) {
+                ret = ptr-str;
+                break;
+            }
+        }
     }
 
     jsstr_release(search_jsstr);
@@ -637,7 +640,7 @@ static HRESULT String_match(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     }
 
     if(is_object_instance(argv[0])) {
-        regexp = iface_to_jsdisp((IUnknown*)get_object(argv[0]));
+        regexp = iface_to_jsdisp(get_object(argv[0]));
         if(regexp && !is_class(regexp, JSCLASS_REGEXP)) {
             jsdisp_release(regexp);
             regexp = NULL;
@@ -800,7 +803,7 @@ static HRESULT String_replace(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
     }
 
     if(is_object_instance(argv[0])) {
-        regexp = iface_to_jsdisp((IUnknown*)get_object(argv[0]));
+        regexp = iface_to_jsdisp(get_object(argv[0]));
         if(regexp && !is_class(regexp, JSCLASS_REGEXP)) {
             jsdisp_release(regexp);
             regexp = NULL;
@@ -817,7 +820,7 @@ static HRESULT String_replace(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
 
     if(argc >= 2) {
         if(is_object_instance(argv[1])) {
-            rep_func = iface_to_jsdisp((IUnknown*)get_object(argv[1]));
+            rep_func = iface_to_jsdisp(get_object(argv[1]));
             if(rep_func && !is_class(rep_func, JSCLASS_FUNCTION)) {
                 jsdisp_release(rep_func);
                 rep_func = NULL;
@@ -1018,7 +1021,7 @@ static HRESULT String_search(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     }
 
     if(is_object_instance(argv[0])) {
-        regexp = iface_to_jsdisp((IUnknown*)get_object(argv[0]));
+        regexp = iface_to_jsdisp(get_object(argv[0]));
         if(regexp && !is_class(regexp, JSCLASS_REGEXP)) {
             jsdisp_release(regexp);
             regexp = NULL;
@@ -1135,7 +1138,7 @@ static HRESULT String_split(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     match_state_t match_result, *match_ptr = &match_result;
     DWORD length, i, match_len = 0;
     const WCHAR *ptr, *ptr2, *str, *match_str = NULL;
-    unsigned limit = UINT32_MAX;
+    unsigned limit = ~0u;
     jsdisp_t *array, *regexp = NULL;
     jsstr_t *jsstr, *match_jsstr, *tmp_str;
     HRESULT hres;
@@ -1162,7 +1165,7 @@ static HRESULT String_split(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     }
 
     if(is_object_instance(argv[0])) {
-        regexp = iface_to_jsdisp((IUnknown*)get_object(argv[0]));
+        regexp = iface_to_jsdisp(get_object(argv[0]));
         if(regexp) {
             if(!is_class(regexp, JSCLASS_REGEXP)) {
                 jsdisp_release(regexp);
@@ -1403,17 +1406,19 @@ static HRESULT String_toLowerCase(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
         return hres;
 
     if(r) {
+        unsigned len = jsstr_length(str);
         jsstr_t *ret;
         WCHAR *buf;
 
-        buf = jsstr_alloc_buf(jsstr_length(str), &ret);
-        if(!buf) {
+        ret = jsstr_alloc_buf(len, &buf);
+        if(!ret) {
             jsstr_release(str);
             return E_OUTOFMEMORY;
         }
 
         jsstr_flush(str, buf);
-        strlwrW(buf);
+        for (; len--; buf++) *buf = tolowerW(*buf);
+
         *r = jsval_string(ret);
     }
     jsstr_release(str);
@@ -1433,17 +1438,19 @@ static HRESULT String_toUpperCase(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
         return hres;
 
     if(r) {
+        unsigned len = jsstr_length(str);
         jsstr_t *ret;
         WCHAR *buf;
 
-        buf = jsstr_alloc_buf(jsstr_length(str), &ret);
-        if(!buf) {
+        ret = jsstr_alloc_buf(len, &buf);
+        if(!ret) {
             jsstr_release(str);
             return E_OUTOFMEMORY;
         }
 
         jsstr_flush(str, buf);
-        struprW(buf);
+        for (; len--; buf++) *buf = toupperW(*buf);
+
         *r = jsval_string(ret);
     }
     jsstr_release(str);
@@ -1471,25 +1478,13 @@ static HRESULT String_localeCompare(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
     return E_NOTIMPL;
 }
 
-static HRESULT String_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
-        jsval_t *r)
+static HRESULT String_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
-    StringInstance *This = string_from_vdisp(jsthis);
+    StringInstance *This = (StringInstance*)jsthis;
 
     TRACE("\n");
 
-    switch(flags) {
-    case INVOKE_FUNC:
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
-    case DISPATCH_PROPERTYGET: {
-        *r = jsval_string(jsstr_addref(This->str));
-        break;
-    }
-    default:
-        FIXME("flags %x\n", flags);
-        return E_NOTIMPL;
-    }
-
+    *r = jsval_string(jsstr_addref(This->str));
     return S_OK;
 }
 
@@ -1544,7 +1539,7 @@ static const builtin_prop_t String_props[] = {
     {indexOfW,               String_indexOf,               PROPF_METHOD|2},
     {italicsW,               String_italics,               PROPF_METHOD},
     {lastIndexOfW,           String_lastIndexOf,           PROPF_METHOD|2},
-    {lengthW,                String_length,                0},
+    {lengthW,                NULL,0,                       String_get_length, String_set_length},
     {linkW,                  String_link,                  PROPF_METHOD|1},
     {localeCompareW,         String_localeCompare,         PROPF_METHOD|1},
     {matchW,                 String_match,                 PROPF_METHOD|1},
@@ -1568,7 +1563,7 @@ static const builtin_prop_t String_props[] = {
 
 static const builtin_info_t String_info = {
     JSCLASS_STRING,
-    {NULL, String_value, 0},
+    {NULL, NULL,0, String_get_value},
     sizeof(String_props)/sizeof(*String_props),
     String_props,
     String_destructor,
@@ -1576,12 +1571,12 @@ static const builtin_info_t String_info = {
 };
 
 static const builtin_prop_t StringInst_props[] = {
-    {lengthW,                String_length,                0}
+    {lengthW,                NULL,0,                       String_get_length, String_set_length}
 };
 
 static const builtin_info_t StringInst_info = {
     JSCLASS_STRING,
-    {NULL, String_value, 0},
+    {NULL, NULL,0, String_get_value},
     sizeof(StringInst_props)/sizeof(*StringInst_props),
     StringInst_props,
     String_destructor,
@@ -1601,8 +1596,8 @@ static HRESULT StringConstr_fromCharCode(script_ctx_t *ctx, vdisp_t *jsthis, WOR
 
     TRACE("\n");
 
-    ret_str = jsstr_alloc_buf(argc, &ret);
-    if(!ret_str)
+    ret = jsstr_alloc_buf(argc, &ret_str);
+    if(!ret)
         return E_OUTOFMEMORY;
 
     for(i=0; i<argc; i++) {
@@ -1699,7 +1694,7 @@ static const builtin_prop_t StringConstr_props[] = {
 
 static const builtin_info_t StringConstr_info = {
     JSCLASS_FUNCTION,
-    {NULL, Function_value, 0},
+    DEFAULT_FUNCTION_VALUE,
     sizeof(StringConstr_props)/sizeof(*StringConstr_props),
     StringConstr_props,
     NULL,
