@@ -37,9 +37,11 @@ typedef SSIZE_T streamsize;
 #define STREAMSIZE_BITS 32
 #endif
 
+void __cdecl _invalid_parameter_noinfo(void);
 void __cdecl _invalid_parameter(const wchar_t*, const wchar_t*,
         const wchar_t*, unsigned int, uintptr_t);
 BOOL __cdecl __uncaught_exception(void);
+int __cdecl _callnewh(size_t);
 
 extern void* (__cdecl *MSVCRT_operator_new)(MSVCP_size_t);
 extern void (__cdecl *MSVCRT_operator_delete)(void*);
@@ -142,6 +144,7 @@ typedef struct
     char null_str;
 } _Yarn_char;
 
+_Yarn_char* __thiscall _Yarn_char_ctor(_Yarn_char*);
 _Yarn_char* __thiscall _Yarn_char_ctor_cstr(_Yarn_char*, const char*);
 _Yarn_char* __thiscall _Yarn_char_copy_ctor(_Yarn_char*, const _Yarn_char*);
 const char* __thiscall _Yarn_char_c_str(const _Yarn_char*);
@@ -255,6 +258,9 @@ wchar_t __thiscall ctype_wchar_widen_ch(const ctype_wchar*, char);
 /* class locale */
 typedef struct
 {
+#if _MSVCP_VER >= 140
+    int unused;
+#endif
     struct _locale__Locimp *ptr;
 } locale;
 
@@ -549,7 +555,9 @@ istreambuf_iterator_wchar *__thiscall num_get_wchar_get_bool(const num_get*, ist
 /* class num_put<wchar> */
 typedef struct {
     locale_facet facet;
+#if _MSVCP_VER < 110
     _Cvtvec cvt;
+#endif
 } num_put;
 
 num_put* num_put_char_use_facet(const locale*);
@@ -631,3 +639,9 @@ static inline int mbstowcs_wrapper( size_t *ret, wchar_t *wcs, size_t size, cons
 #endif
 
 void free_misc(void);
+
+#if _MSVCP_VER >= 140
+#define UCRTBASE_PRINTF_STANDARD_SNPRINTF_BEHAVIOUR      (0x0002)
+int __cdecl __stdio_common_vsprintf(unsigned __int64 options, char *str, size_t len, const char *format,
+                                    _locale_t locale, __ms_va_list valist);
+#endif
