@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2007 Alexandre Julliard
  * Copyright (C) 2010 Damjan Jovanovic
+ * Copyright (C) 2016 Sebastian Lackner
+ * Copyright (C) 2016 CodeWeavers, Aric Stewart
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,6 +32,7 @@
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
+#include "winsvc.h"
 #include "winternl.h"
 #include "excpt.h"
 #include "winioctl.h"
@@ -45,6 +48,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(ntoskrnl);
 WINE_DECLARE_DEBUG_CHANNEL(relay);
+WINE_DECLARE_DEBUG_CHANNEL(plugplay);
 
 BOOLEAN KdDebuggerEnabled = FALSE;
 ULONG InitSafeBootMode = 0;
@@ -66,6 +70,15 @@ KSERVICE_TABLE_DESCRIPTOR KeServiceDescriptorTable[4] = { { 0 } };
 typedef void (WINAPI *PCREATE_PROCESS_NOTIFY_ROUTINE)(HANDLE,HANDLE,BOOLEAN);
 typedef void (WINAPI *PCREATE_THREAD_NOTIFY_ROUTINE)(HANDLE,HANDLE,BOOLEAN);
 
+static const WCHAR servicesW[] = {'\\','R','e','g','i','s','t','r','y',
+                                  '\\','M','a','c','h','i','n','e',
+                                  '\\','S','y','s','t','e','m',
+                                  '\\','C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t',
+                                  '\\','S','e','r','v','i','c','e','s',
+                                  '\\',0};
+
+#define MAX_SERVICE_NAME 260
+
 /* tid of the thread running client request */
 static DWORD request_thread;
 
@@ -81,7 +94,15 @@ struct wine_driver
     DRIVER_EXTENSION driver_extension;
 };
 
-static struct wine_rb_tree wine_drivers;
+static int wine_drivers_rb_compare( const void *key, const struct wine_rb_entry *entry )
+{
+    const struct wine_driver *driver = WINE_RB_ENTRY_VALUE( entry, const struct wine_driver, entry );
+    const UNICODE_STRING *k = key;
+
+    return RtlCompareUnicodeString( k, &driver->driver_obj.DriverName, FALSE );
+}
+
+static struct wine_rb_tree wine_drivers = { wine_drivers_rb_compare };
 
 static CRITICAL_SECTION drivers_cs;
 static CRITICAL_SECTION_DEBUG critsect_debug =
@@ -92,37 +113,6 @@ static CRITICAL_SECTION_DEBUG critsect_debug =
 };
 static CRITICAL_SECTION drivers_cs = { &critsect_debug, -1, 0, 0, 0, 0 };
 
-static void *wine_drivers_rb_alloc( size_t size )
-{
-    return HeapAlloc( GetProcessHeap(), 0, size );
-}
-
-static void *wine_drivers_rb_realloc( void *ptr, size_t size )
-{
-    return HeapReAlloc( GetProcessHeap(), 0, ptr, size );
-}
-
-static void wine_drivers_rb_free( void *ptr )
-{
-    HeapFree( GetProcessHeap(), 0, ptr );
-}
-
-static int wine_drivers_rb_compare( const void *key, const struct wine_rb_entry *entry )
-{
-    const struct wine_driver *driver = WINE_RB_ENTRY_VALUE( entry, const struct wine_driver, entry );
-    const UNICODE_STRING *k = key;
-
-    return RtlCompareUnicodeString( k, &driver->driver_obj.DriverName, FALSE );
-}
-
-static const struct wine_rb_functions wine_drivers_rb_functions =
-{
-    wine_drivers_rb_alloc,
-    wine_drivers_rb_realloc,
-    wine_drivers_rb_free,
-    wine_drivers_rb_compare,
-};
-
 #ifdef __i386__
 #define DEFINE_FASTCALL1_ENTRYPOINT( name ) \
     __ASM_STDCALL_FUNC( name, 4, \
@@ -871,12 +861,6 @@ PIRP WINAPI IoBuildSynchronousFsdRequest(ULONG majorfunc, PDEVICE_OBJECT device,
 static void build_driver_keypath( const WCHAR *name, UNICODE_STRING *keypath )
 {
     static const WCHAR driverW[] = {'\\','D','r','i','v','e','r','\\',0};
-    static const WCHAR servicesW[] = {'\\','R','e','g','i','s','t','r','y',
-                                      '\\','M','a','c','h','i','n','e',
-                                      '\\','S','y','s','t','e','m',
-                                      '\\','C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t',
-                                      '\\','S','e','r','v','i','c','e','s',
-                                      '\\',0};
     WCHAR *str;
 
     /* Check what prefix is present */
@@ -952,7 +936,7 @@ void WINAPI IoDeleteDriver( DRIVER_OBJECT *driver_object )
     TRACE( "(%p)\n", driver_object );
 
     EnterCriticalSection( &drivers_cs );
-    wine_rb_remove( &wine_drivers, &driver_object->DriverName );
+    wine_rb_remove_key( &wine_drivers, &driver_object->DriverName );
     LeaveCriticalSection( &drivers_cs );
 
     RtlFreeUnicodeString( &driver_object->DriverName );
@@ -1124,13 +1108,19 @@ NTSTATUS  WINAPI IoGetDeviceObjectPointer( UNICODE_STRING *name, ACCESS_MASK acc
 /***********************************************************************
  *           IoGetAttachedDevice   (NTOSKRNL.EXE.@)
  */
-PDEVICE_OBJECT WINAPI IoGetAttachedDevice( PDEVICE_OBJECT device )
+DEVICE_OBJECT* WINAPI IoGetAttachedDevice( DEVICE_OBJECT *device )
 {
-    FIXME( "stub: %p\n", device );
+    DEVICE_OBJECT *result = device;
 
-    return device;
+    TRACE( "(%p)\n", device );
+
+    while (result->AttachedDevice)
+        result = result->AttachedDevice;
+
+    return result;
 }
 
+
 /***********************************************************************
  *           IoGetDeviceProperty   (NTOSKRNL.EXE.@)
  */
@@ -2140,6 +2130,21 @@ NTSTATUS WINAPI ObReferenceObjectByName( UNICODE_STRING *ObjectName,
 }
 
 
+static void ObReferenceObject( void *obj )
+{
+    TRACE( "(%p): stub\n", obj );
+}
+
+
+/***********************************************************************
+ *           ObDereferenceObject   (NTOSKRNL.EXE.@)
+ */
+void WINAPI ObDereferenceObject( void *obj )
+{
+    TRACE( "(%p): stub\n", obj );
+}
+
+
 /***********************************************************************
  *           ObfReferenceObject   (NTOSKRNL.EXE.@)
  */
@@ -2150,7 +2155,7 @@ void WINAPI __regs_ObfReferenceObject( void *obj )
 void WINAPI ObfReferenceObject( void *obj )
 #endif
 {
-    FIXME( "(%p): stub\n", obj );
+    ObReferenceObject( obj );
 }
 
 
@@ -2164,7 +2169,18 @@ void WINAPI __regs_ObfDereferenceObject( void *obj )
 void WINAPI ObfDereferenceObject( void *obj )
 #endif
 {
-    FIXME( "(%p): stub\n", obj );
+    ObDereferenceObject( obj );
+}
+
+
+/***********************************************************************
+ *           IoGetAttachedDeviceReference   (NTOSKRNL.EXE.@)
+ */
+DEVICE_OBJECT* WINAPI IoGetAttachedDeviceReference( DEVICE_OBJECT *device )
+{
+    DEVICE_OBJECT *result = IoGetAttachedDevice( device );
+    ObReferenceObject( result );
+    return result;
 }
 
 
@@ -2435,7 +2451,6 @@ BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, LPVOID reserved )
     {
     case DLL_PROCESS_ATTACH:
         DisableThreadLibraryCalls( inst );
-        if (wine_rb_init( &wine_drivers, &wine_drivers_rb_functions )) return FALSE;
 #if defined(__i386__) || defined(__x86_64__)
         handler = RtlAddVectoredExceptionHandler( TRUE, vectored_handler );
 #endif
@@ -2632,3 +2647,367 @@ NTSTATUS WINAPI IoAttachDevice(DEVICE_OBJECT *source, UNICODE_STRING *target, DE
     FIXME("(%p, %s, %p): stub\n", source, debugstr_us(target), attached);
     return STATUS_NOT_IMPLEMENTED;
 }
+
+
+static NTSTATUS open_driver( const UNICODE_STRING *service_name, SC_HANDLE *service )
+{
+    QUERY_SERVICE_CONFIGW *service_config = NULL;
+    SC_HANDLE manager_handle;
+    DWORD config_size = 0;
+    WCHAR *name;
+
+    if (!(name = RtlAllocateHeap( GetProcessHeap(), 0, service_name->Length + sizeof(WCHAR) )))
+        return STATUS_NO_MEMORY;
+
+    memcpy( name, service_name->Buffer, service_name->Length );
+    name[ service_name->Length / sizeof(WCHAR) ] = 0;
+
+    if (strncmpW( name, servicesW, strlenW(servicesW) ))
+    {
+        FIXME( "service name %s is not a keypath\n", debugstr_us(service_name) );
+        RtlFreeHeap( GetProcessHeap(), 0, name );
+        return STATUS_NOT_IMPLEMENTED;
+    }
+
+    if (!(manager_handle = OpenSCManagerW( NULL, NULL, SC_MANAGER_CONNECT )))
+    {
+        WARN( "failed to connect to service manager\n" );
+        RtlFreeHeap( GetProcessHeap(), 0, name );
+        return STATUS_NOT_SUPPORTED;
+    }
+
+    *service = OpenServiceW( manager_handle, name + strlenW(servicesW), SERVICE_ALL_ACCESS );
+    RtlFreeHeap( GetProcessHeap(), 0, name );
+    CloseServiceHandle( manager_handle );
+
+    if (!*service)
+    {
+        WARN( "failed to open service %s\n", debugstr_us(service_name) );
+        return STATUS_UNSUCCESSFUL;
+    }
+
+    QueryServiceConfigW( *service, NULL, 0, &config_size );
+    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
+    {
+        WARN( "failed to query service config\n" );
+        goto error;
+    }
+
+    if (!(service_config = RtlAllocateHeap( GetProcessHeap(), 0, config_size )))
+        goto error;
+
+    if (!QueryServiceConfigW( *service, service_config, config_size, &config_size ))
+    {
+        WARN( "failed to query service config\n" );
+        goto error;
+    }
+
+    if (service_config->dwServiceType != SERVICE_KERNEL_DRIVER &&
+        service_config->dwServiceType != SERVICE_FILE_SYSTEM_DRIVER)
+    {
+        WARN( "service %s is not a kernel driver\n", debugstr_us(service_name) );
+        goto error;
+    }
+
+    TRACE( "opened service for driver %s\n", debugstr_us(service_name) );
+    RtlFreeHeap( GetProcessHeap(), 0, service_config );
+    return STATUS_SUCCESS;
+
+error:
+    CloseServiceHandle( *service );
+    RtlFreeHeap( GetProcessHeap(), 0, service_config );
+    return STATUS_UNSUCCESSFUL;
+}
+
+
+/***********************************************************************
+ *           ZwLoadDriver (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI ZwLoadDriver( const UNICODE_STRING *service_name )
+{
+    SERVICE_STATUS_PROCESS service_status;
+    SC_HANDLE service_handle;
+    ULONGLONG start_time;
+    NTSTATUS status;
+    DWORD bytes;
+
+    TRACE( "(%s)\n", debugstr_us(service_name) );
+
+    if ((status = open_driver( service_name, &service_handle )) != STATUS_SUCCESS)
+        return status;
+
+    TRACE( "trying to start %s\n", debugstr_us(service_name) );
+
+    start_time = GetTickCount64();
+    for (;;)
+    {
+        if (StartServiceW( service_handle, 0, NULL )) break;
+        if (GetLastError() == ERROR_SERVICE_ALREADY_RUNNING) break;
+        if (GetLastError() != ERROR_SERVICE_DATABASE_LOCKED) goto error;
+        if (GetTickCount64() - start_time > 30000) goto error;
+        Sleep( 100 );
+    }
+
+    start_time = GetTickCount64();
+    for (;;)
+    {
+        if (!QueryServiceStatusEx( service_handle, SC_STATUS_PROCESS_INFO,
+                                   (BYTE *)&service_status, sizeof(service_status), &bytes )) goto error;
+        if (service_status.dwCurrentState != SERVICE_START_PENDING) break;
+        if (GetTickCount64() - start_time > 30000) goto error;
+        Sleep( 100 );
+    }
+
+    if (service_status.dwCurrentState == SERVICE_RUNNING)
+    {
+        if (service_status.dwProcessId != GetCurrentProcessId())
+            FIXME( "driver %s was loaded into a different process\n", debugstr_us(service_name) );
+
+        status = STATUS_SUCCESS;
+        goto done;
+    }
+
+error:
+    WARN( "failed to start service %s\n", debugstr_us(service_name) );
+    status = STATUS_UNSUCCESSFUL;
+
+done:
+    TRACE( "returning status %08x\n", status );
+    CloseServiceHandle( service_handle );
+    return status;
+}
+
+
+/***********************************************************************
+ *           ZwUnloadDriver (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI ZwUnloadDriver( const UNICODE_STRING *service_name )
+{
+    SERVICE_STATUS service_status;
+    SC_HANDLE service_handle;
+    ULONGLONG start_time;
+    NTSTATUS status;
+
+    TRACE( "(%s)\n", debugstr_us(service_name) );
+
+    if ((status = open_driver( service_name, &service_handle )) != STATUS_SUCCESS)
+        return status;
+
+    if (!ControlService( service_handle, SERVICE_CONTROL_STOP, &service_status ))
+        goto error;
+
+    start_time = GetTickCount64();
+    for (;;)
+    {
+        if (!QueryServiceStatus( service_handle, &service_status )) goto error;
+        if (service_status.dwCurrentState != SERVICE_STOP_PENDING) break;
+        if (GetTickCount64() - start_time > 30000) goto error;
+        Sleep( 100 );
+    }
+
+    if (service_status.dwCurrentState == SERVICE_STOPPED)
+    {
+        status = STATUS_SUCCESS;
+        goto done;
+    }
+
+error:
+    WARN( "failed to stop service %s\n", debugstr_us(service_name) );
+    status = STATUS_UNSUCCESSFUL;
+
+done:
+    TRACE( "returning status %08x\n", status );
+    CloseServiceHandle( service_handle );
+    return status;
+}
+
+
+static NTSTATUS WINAPI internal_complete( DEVICE_OBJECT *device, IRP *irp, void *context )
+{
+    SetEvent( irp->UserEvent );
+    return STATUS_MORE_PROCESSING_REQUIRED;
+}
+
+
+static NTSTATUS send_device_irp( DEVICE_OBJECT *device, IRP *irp, ULONG_PTR *info )
+{
+    NTSTATUS status;
+    IO_STACK_LOCATION *irpsp;
+    HANDLE event = CreateEventA( NULL, FALSE, FALSE, NULL );
+    DEVICE_OBJECT *toplevel_device;
+
+    irp->UserEvent = event;
+    irp->IoStatus.u.Status = STATUS_NOT_SUPPORTED;
+    irpsp = IoGetNextIrpStackLocation( irp );
+    irpsp->CompletionRoutine = internal_complete;
+    irpsp->Control = SL_INVOKE_ON_SUCCESS | SL_INVOKE_ON_ERROR | SL_INVOKE_ON_CANCEL;
+
+    toplevel_device = IoGetAttachedDeviceReference( device );
+    status = IoCallDriver( toplevel_device, irp );
+
+    if (status == STATUS_PENDING)
+        WaitForSingleObject( event, INFINITE );
+
+    status = irp->IoStatus.u.Status;
+    if (info)
+        *info = irp->IoStatus.Information;
+    IoCompleteRequest( irp, IO_NO_INCREMENT );
+    ObDereferenceObject( toplevel_device );
+    CloseHandle( event );
+    return status;
+}
+
+
+static NTSTATUS get_device_id( DEVICE_OBJECT *device, BUS_QUERY_ID_TYPE type, WCHAR **id )
+{
+    IO_STACK_LOCATION *irpsp;
+    IO_STATUS_BLOCK irp_status;
+    IRP *irp;
+
+    if (!(irp = IoBuildSynchronousFsdRequest( IRP_MJ_PNP, device, NULL, 0, NULL, NULL, &irp_status )))
+        return STATUS_NO_MEMORY;
+
+    irpsp = IoGetNextIrpStackLocation( irp );
+    irpsp->MinorFunction = IRP_MN_QUERY_ID;
+    irpsp->Parameters.QueryId.IdType = type;
+
+    return send_device_irp( device, irp, (ULONG_PTR *)id );
+}
+
+
+static BOOL get_driver_for_id( const WCHAR *id, WCHAR *driver )
+{
+    static const WCHAR serviceW[] = {'S','e','r','v','i','c','e',0};
+    static const UNICODE_STRING service_str = { sizeof(serviceW) - sizeof(WCHAR), sizeof(serviceW), (WCHAR *)serviceW };
+    static const WCHAR critical_fmtW[] =
+        {'\\','R','e','g','i','s','t','r','y',
+         '\\','M','a','c','h','i','n','e',
+         '\\','S','y','s','t','e','m',
+         '\\','C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t',
+         '\\','C','o','n','t','r','o','l',
+         '\\','C','r','i','t','i','c','a','l','D','e','v','i','c','e','D','a','t','a','b','a','s','e',
+         '\\','%','s',0};
+    WCHAR buffer[FIELD_OFFSET( KEY_VALUE_PARTIAL_INFORMATION, Data[MAX_SERVICE_NAME * sizeof(WCHAR)] )];
+    KEY_VALUE_PARTIAL_INFORMATION *info = (KEY_VALUE_PARTIAL_INFORMATION *)buffer;
+    OBJECT_ATTRIBUTES attr;
+    UNICODE_STRING key;
+    NTSTATUS status;
+    HANDLE hkey;
+    WCHAR *keyW;
+    DWORD len;
+
+    if (!(keyW = RtlAllocateHeap( GetProcessHeap(), 0, sizeof(critical_fmtW) + strlenW(id) * sizeof(WCHAR) )))
+        return STATUS_NO_MEMORY;
+
+    sprintfW( keyW, critical_fmtW, id );
+    RtlInitUnicodeString( &key, keyW );
+    InitializeObjectAttributes( &attr, &key, OBJ_CASE_INSENSITIVE | OBJ_KERNEL_HANDLE, NULL, NULL );
+
+    status = NtOpenKey( &hkey, KEY_ALL_ACCESS, &attr );
+    RtlFreeUnicodeString( &key );
+    if (status != STATUS_SUCCESS)
+    {
+        TRACE_(plugplay)( "no driver found for %s\n", debugstr_w(id) );
+        return FALSE;
+    }
+
+    status = NtQueryValueKey( hkey, &service_str, KeyValuePartialInformation,
+                              info, sizeof(buffer) - sizeof(WCHAR), &len );
+    NtClose( hkey );
+    if (status != STATUS_SUCCESS || info->Type != REG_SZ)
+    {
+        TRACE_(plugplay)( "no driver found for %s\n", debugstr_w(id) );
+        return FALSE;
+    }
+
+    memcpy( driver, info->Data, info->DataLength );
+    driver[ info->DataLength / sizeof(WCHAR) ] = 0;
+    TRACE_(plugplay)( "found driver %s for %s\n", debugstr_w(driver), debugstr_w(id) );
+    return TRUE;
+}
+
+
+static void handle_bus_relations( DEVICE_OBJECT *device )
+{
+    static const WCHAR driverW[] = {'\\','D','r','i','v','e','r','\\',0};
+    WCHAR buffer[MAX_SERVICE_NAME + sizeof(servicesW)/sizeof(WCHAR)];
+    WCHAR driver[MAX_SERVICE_NAME] = {0};
+    DRIVER_OBJECT *driver_obj;
+    UNICODE_STRING string;
+    WCHAR *ids, *ptr;
+    NTSTATUS status;
+
+    TRACE_(plugplay)( "(%p)\n", device );
+
+    /* We could (should?) do a full IRP_MN_QUERY_DEVICE_RELATIONS query,
+     * but we don't have to, we have the DEVICE_OBJECT of the new device
+     * so we can simply handle the process here */
+
+    status = get_device_id( device, BusQueryCompatibleIDs, &ids );
+    if (status != STATUS_SUCCESS || !ids)
+    {
+        ERR_(plugplay)( "Failed to get device IDs\n" );
+        return;
+    }
+
+    for (ptr = ids; *ptr; ptr += strlenW(ptr) + 1)
+    {
+        if (get_driver_for_id( ptr, driver ))
+            break;
+    }
+    RtlFreeHeap( GetProcessHeap(), 0, ids );
+
+    if (!driver[0])
+    {
+        ERR_(plugplay)( "No matching driver found for device\n" );
+        return;
+    }
+
+    strcpyW( buffer, servicesW );
+    strcatW( buffer, driver );
+    RtlInitUnicodeString( &string, buffer );
+    if (ZwLoadDriver( &string ) != STATUS_SUCCESS)
+    {
+        ERR_(plugplay)( "Failed to load driver %s\n", debugstr_w(driver) );
+        return;
+    }
+
+    strcpyW( buffer, driverW );
+    strcatW( buffer, driver );
+    RtlInitUnicodeString( &string, buffer );
+    if (ObReferenceObjectByName( &string, OBJ_CASE_INSENSITIVE, NULL,
+                                 0, NULL, KernelMode, NULL, (void **)&driver_obj ) != STATUS_SUCCESS)
+    {
+        ERR_(plugplay)( "Failed to locate loaded driver %s\n", debugstr_w(driver) );
+        return;
+    }
+
+    if (driver_obj->DriverExtension->AddDevice)
+        status = driver_obj->DriverExtension->AddDevice( driver_obj, device );
+    else
+        status = STATUS_NOT_IMPLEMENTED;
+
+    ObDereferenceObject( driver_obj );
+
+    if (status != STATUS_SUCCESS)
+        ERR_(plugplay)( "AddDevice failed for driver %s\n", debugstr_w(driver) );
+}
+
+
+/***********************************************************************
+ *           IoInvalidateDeviceRelations (NTOSKRNL.EXE.@)
+ */
+void WINAPI IoInvalidateDeviceRelations( DEVICE_OBJECT *device_object, DEVICE_RELATION_TYPE type )
+{
+    TRACE( "(%p, %i)\n", device_object, type );
+
+    switch (type)
+    {
+        case BusRelations:
+            handle_bus_relations( device_object );
+            break;
+        default:
+            FIXME( "unhandled relation %i\n", type );
+            break;
+    }
+}
