@@ -117,6 +117,8 @@ typedef struct tagUSER_DRIVER {
     void   (CDECL *pWindowPosChanged)(HWND,HWND,UINT,const RECT *,const RECT *,const RECT *,const RECT *,struct window_surface*);
     /* system parameters */
     BOOL   (CDECL *pSystemParametersInfo)(UINT,UINT,void*,UINT);
+    /* thread management */
+    void   (CDECL *pThreadDetach)(void);
 } USER_DRIVER;
 
 extern const USER_DRIVER *USER_Driver DECLSPEC_HIDDEN;
@@ -194,6 +196,8 @@ struct user_thread_info
 C_ASSERT( sizeof(struct user_thread_info) <= sizeof(((TEB *)0)->Win32ClientInfo) );
 
 extern INT global_key_state_counter DECLSPEC_HIDDEN;
+extern BOOL (WINAPI *imm_register_window)(HWND) DECLSPEC_HIDDEN;
+extern void (WINAPI *imm_unregister_window)(HWND) DECLSPEC_HIDDEN;
 
 struct user_key_state_info
 {
@@ -339,6 +343,7 @@ typedef struct
 
 #include "poppack.h"
 
+extern int bitmap_info_size( const BITMAPINFO * info, WORD coloruse ) DECLSPEC_HIDDEN;
 extern BOOL get_icon_size( HICON handle, SIZE *size ) DECLSPEC_HIDDEN;
 
 /* Mingw's assert() imports MessageBoxA and gets confused by user32 exporting it */
