@@ -465,6 +465,7 @@ static void test_WsWriteStartAttribute(void)
     text.text.textType = WS_XML_TEXT_TYPE_UTF8;
     text.value.length  = 1;
     text.value.bytes   = (BYTE *)"0";
+    text.value.dictionary = NULL;
     hr = WsWriteText( writer, &text.text, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
     check_output( writer, "", __LINE__ );
@@ -577,9 +578,13 @@ static void test_WsWriteType(void)
 
 static void test_basic_type(void)
 {
+    static WCHAR testW[] = {'t','e','s','t',0};
     HRESULT hr;
     WS_XML_WRITER *writer;
-    WS_XML_STRING localname = {1, (BYTE *)"t"}, ns = {0, NULL};
+    WS_XML_STRING localname = {1, (BYTE *)"t"}, ns = {0, NULL}, xmlstr;
+    GUID guid;
+    WCHAR *str;
+    WS_STRING string;
     ULONG i;
     static const struct
     {
@@ -671,6 +676,68 @@ static void test_basic_type(void)
         check_output( writer, tests[i].result2, __LINE__ );
     }
 
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    memset( &guid, 0, sizeof(guid) );
+    hr = WsWriteType( writer, WS_ELEMENT_TYPE_MAPPING, WS_GUID_TYPE, NULL, WS_WRITE_REQUIRED_VALUE,
+                      &guid, sizeof(guid), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<t>00000000-0000-0000-0000-000000000000</t>", __LINE__ );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    string.chars  = testW;
+    string.length = 4;
+    hr = WsWriteType( writer, WS_ELEMENT_TYPE_MAPPING, WS_STRING_TYPE, NULL, WS_WRITE_REQUIRED_VALUE,
+                      &string, sizeof(string), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<t>test</t>", __LINE__ );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    str = testW;
+    hr = WsWriteType( writer, WS_ELEMENT_TYPE_MAPPING, WS_WSZ_TYPE, NULL, WS_WRITE_REQUIRED_POINTER,
+                      &str, sizeof(str), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<t>test</t>", __LINE__ );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    xmlstr.bytes  = (BYTE *)"test";
+    xmlstr.length = 4;
+    hr = WsWriteType( writer, WS_ELEMENT_TYPE_MAPPING, WS_XML_STRING_TYPE, NULL, WS_WRITE_REQUIRED_VALUE,
+                      &xmlstr, sizeof(xmlstr), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<t>test</t>", __LINE__ );
+
     WsFreeWriter( writer );
 }
 
@@ -725,6 +792,21 @@ static void test_simple_struct_type(void)
     ok( hr == S_OK, "got %08x\n", hr );
     check_output( writer, "<struct>value</struct>", __LINE__ );
 
+    /* required value */
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteType( writer, WS_ELEMENT_CONTENT_TYPE_MAPPING, WS_STRUCT_TYPE, &s,
+                      WS_WRITE_REQUIRED_VALUE, test, sizeof(*test), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<struct>value</struct>", __LINE__ );
+
     hr = set_output( writer );
     ok( hr == S_OK, "got %08x\n", hr );
 
@@ -1049,6 +1131,7 @@ static void test_WsWriteStartCData(void)
     text.text.textType = WS_XML_TEXT_TYPE_UTF8;
     text.value.bytes = (BYTE *)"<data>";
     text.value.length = 6;
+    text.value.dictionary = NULL;
     hr = WsWriteText( writer, &text.text, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
     check_output( writer, "<t><![CDATA[<data>", __LINE__ );
@@ -1576,6 +1659,7 @@ static void test_WsGetWriterPosition(void)
 
 static void test_WsSetWriterPosition(void)
 {
+    WS_XML_STRING localname = {1, (BYTE *)"t"}, ns = {0, NULL};
     WS_HEAP *heap;
     WS_XML_WRITER *writer;
     WS_XML_BUFFER *buf1, *buf2;
@@ -1617,6 +1701,29 @@ static void test_WsSetWriterPosition(void)
     hr = WsSetWriterPosition( writer, &pos, NULL );
     ok( hr == E_INVALIDARG, "got %08x\n", hr );
 
+    hr = WsSetOutputToBuffer( writer, buf1, NULL, 0, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    /* try to write at non-final position */
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    pos.buffer = pos.node = NULL;
+    hr = WsGetWriterPosition( writer, &pos, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    ok( pos.buffer == buf1, "wrong buffer\n" );
+    ok( pos.node != NULL, "node not set\n" );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output_buffer( buf1, "<t/>", __LINE__ );
+
+    hr = WsSetWriterPosition( writer, &pos, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    todo_wine ok( hr == WS_E_INVALID_FORMAT, "got %08x\n", hr );
+
     WsFreeWriter( writer );
     WsFreeHeap( heap );
 }
@@ -1713,6 +1820,7 @@ static void test_WsWriteNode(void)
     utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
     utf8.value.bytes   = (BYTE *)"value";
     utf8.value.length  = sizeof("value") - 1;
+    utf8.value.dictionary = NULL;
 
     attr.singleQuote = TRUE;
     attr.isXmlNs     = FALSE;
@@ -1735,6 +1843,7 @@ static void test_WsWriteNode(void)
     comment.node.nodeType = WS_XML_NODE_TYPE_COMMENT;
     comment.value.bytes   = (BYTE *)"comment";
     comment.value.length  = sizeof("comment") - 1;
+    comment.value.dictionary = NULL;
     hr = WsWriteNode( writer, (const WS_XML_NODE *)&comment, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
 
@@ -1752,6 +1861,7 @@ static void test_WsWriteNode(void)
 
     utf8.value.bytes   = (BYTE *)"cdata";
     utf8.value.length  = sizeof("cdata") - 1;
+    utf8.value.dictionary = NULL;
     text.node.nodeType = WS_XML_NODE_TYPE_TEXT;
     text.text          = &utf8.text;
     hr = WsWriteNode( writer, (const WS_XML_NODE *)&text, NULL );
@@ -1763,6 +1873,7 @@ static void test_WsWriteNode(void)
 
     utf8.value.bytes   = (BYTE *)"text";
     utf8.value.length  = sizeof("text") - 1;
+    utf8.value.dictionary = NULL;
     hr = WsWriteNode( writer, (const WS_XML_NODE *)&text, NULL );
     ok( hr == S_OK, "got %08x\n", hr );
 
@@ -2006,6 +2117,198 @@ static void test_double(void)
     WsFreeWriter( writer );
 }
 
+static void test_field_flags(void)
+{
+    static const char expected[] =
+        "<t><bool a:nil=\"true\" xmlns:a=\"http://www.w3.org/2001/XMLSchema-instance\"/><int32>-1</int32>"
+        "<xmlstr a:nil=\"true\" xmlns:a=\"http://www.w3.org/2001/XMLSchema-instance\"/></t>";
+    HRESULT hr;
+    WS_XML_WRITER *writer;
+    WS_STRUCT_DESCRIPTION s;
+    WS_FIELD_DESCRIPTION f, f2, f3, f4, *fields[4];
+    WS_XML_STRING localname = {1, (BYTE *)"t"}, ns = {0, NULL}, str_guid = {4, (BYTE *)"guid"};
+    WS_XML_STRING str_int32 = {5, (BYTE *)"int32"}, str_bool = {4, (BYTE *)"bool"};
+    WS_XML_STRING str_xmlstr = {6, (BYTE *)"xmlstr"};
+    INT32 val = -1;
+    struct test
+    {
+        GUID           guid;
+        BOOL          *bool_ptr;
+        INT32         *int32_ptr;
+        WS_XML_STRING  xmlstr;
+    } test;
+
+    hr = WsCreateWriter( NULL, 0, &writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    memset( &f, 0, sizeof(f) );
+    f.mapping   = WS_ELEMENT_FIELD_MAPPING;
+    f.localName = &str_guid;
+    f.ns        = &ns;
+    f.type      = WS_GUID_TYPE;
+    f.options   = WS_FIELD_OPTIONAL;
+    fields[0] = &f;
+
+    memset( &f2, 0, sizeof(f2) );
+    f2.mapping   = WS_ELEMENT_FIELD_MAPPING;
+    f2.localName = &str_bool;
+    f2.offset    = FIELD_OFFSET(struct test, bool_ptr);
+    f2.ns        = &ns;
+    f2.type      = WS_BOOL_TYPE;
+    f2.options   = WS_FIELD_POINTER|WS_FIELD_NILLABLE;
+    fields[1] = &f2;
+
+    memset( &f3, 0, sizeof(f3) );
+    f3.mapping   = WS_ELEMENT_FIELD_MAPPING;
+    f3.localName = &str_int32;
+    f3.offset    = FIELD_OFFSET(struct test, int32_ptr);
+    f3.ns        = &ns;
+    f3.type      = WS_INT32_TYPE;
+    f3.options   = WS_FIELD_POINTER|WS_FIELD_NILLABLE;
+    fields[2] = &f3;
+
+    memset( &f4, 0, sizeof(f4) );
+    f4.mapping   = WS_ELEMENT_FIELD_MAPPING;
+    f4.localName = &str_xmlstr;
+    f4.offset    = FIELD_OFFSET(struct test, xmlstr);
+    f4.ns        = &ns;
+    f4.type      = WS_XML_STRING_TYPE;
+    f4.options   = WS_FIELD_NILLABLE;
+    fields[3] = &f4;
+
+    memset( &s, 0, sizeof(s) );
+    s.size       = sizeof(struct test);
+    s.alignment  = TYPE_ALIGNMENT(struct test);
+    s.fields     = fields;
+    s.fieldCount = 4;
+
+    memset( &test, 0, sizeof(test) );
+    test.int32_ptr = &val;
+    hr = WsWriteType( writer, WS_ELEMENT_TYPE_MAPPING, WS_STRUCT_TYPE, &s, WS_WRITE_REQUIRED_VALUE,
+                      &test, sizeof(test), NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, expected, __LINE__ );
+
+    WsFreeWriter( writer );
+}
+
+static void test_WsWriteText(void)
+{
+    HRESULT hr;
+    WS_XML_WRITER *writer;
+    WS_XML_UTF8_TEXT utf8;
+
+    hr = WsCreateWriter( NULL, 0, &writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes  = (BYTE *)"test";
+    utf8.value.length = 4;
+    utf8.value.dictionary = NULL;
+    hr = WsWriteText( writer, &utf8.text, NULL );
+    todo_wine ok( hr == WS_E_INVALID_FORMAT, "got %08x\n", hr );
+
+    WsFreeWriter( writer );
+}
+
+static void test_WsWriteArray(void)
+{
+    static const WS_XML_STRING localname = {4, (BYTE *)"item"}, localname2 = {5, (BYTE *)"array"};
+    static const WS_XML_STRING ns = {0, NULL};
+    WS_XML_WRITER *writer;
+    BOOL array_bool[2];
+    HRESULT hr;
+
+    hr = WsCreateWriter( NULL, 0, &writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteArray( writer, NULL, NULL, 0, NULL, 0, 0, 0, NULL );
+    ok( hr == WS_E_INVALID_OPERATION, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+    hr = WsWriteArray( writer, NULL, NULL, 0, NULL, 0, 0, 0, NULL );
+    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+    hr = WsWriteArray( writer, &localname, NULL, 0, NULL, 0, 0, 0, NULL );
+    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+    hr = WsWriteArray( writer, &localname, &ns, 0, NULL, 0, 0, 0, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "", __LINE__ );
+
+    hr = WsWriteArray( writer, &localname, &ns, ~0u, NULL, 0, 0, 0, NULL );
+    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, NULL, 0, 0, 0, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "", __LINE__ );
+
+    array_bool[0] = FALSE;
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, array_bool, 0, 0, 0, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "", __LINE__ );
+
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, array_bool, sizeof(array_bool), 0, 0, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "", __LINE__ );
+
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, NULL, sizeof(array_bool), 0, 0, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "", __LINE__ );
+
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, NULL, sizeof(array_bool), 0, 1, NULL );
+    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, array_bool, sizeof(array_bool), 0, 1, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<item>false</item>", __LINE__ );
+
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, array_bool, sizeof(array_bool) - 1, 0, 2, NULL );
+    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, array_bool, sizeof(array_bool), 0, 3, NULL );
+    ok( hr == E_INVALIDARG, "got %08x\n", hr );
+
+    hr = set_output( writer );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteStartElement( writer, NULL, &localname2, &ns, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    array_bool[1] = TRUE;
+    hr = WsWriteArray( writer, &localname, &ns, WS_BOOL_VALUE_TYPE, array_bool, sizeof(array_bool), 0, 2, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+
+    hr = WsWriteEndElement( writer, NULL );
+    ok( hr == S_OK, "got %08x\n", hr );
+    check_output( writer, "<array><item>false</item><item>true</item></array>", __LINE__ );
+
+    WsFreeWriter( writer );
+}
+
 START_TEST(writer)
 {
     test_WsCreateWriter();
@@ -2032,4 +2335,7 @@ START_TEST(writer)
     test_WsCopyNode();
     test_text_types();
     test_double();
+    test_field_flags();
+    test_WsWriteText();
+    test_WsWriteArray();
 }
