@@ -471,9 +471,10 @@ static const WCHAR LOCALE_NAME_SYSTEM_DEFAULT[] = {'!','s','y','s','-','d','e','
 #define MAX_DEFAULTCHAR   2
 
 /* Defines for calendar handling */
-#define CAL_NOUSEROVERRIDE     LOCALE_NOUSEROVERRIDE
-#define CAL_USE_CP_ACP         LOCALE_USE_CP_ACP
-#define CAL_RETURN_NUMBER      LOCALE_RETURN_NUMBER
+#define CAL_NOUSEROVERRIDE        LOCALE_NOUSEROVERRIDE
+#define CAL_USE_CP_ACP            LOCALE_USE_CP_ACP
+#define CAL_RETURN_NUMBER         LOCALE_RETURN_NUMBER
+#define CAL_RETURN_GENITIVE_NAMES LOCALE_RETURN_GENITIVE_NAMES
 
 #define CAL_ICALINTVALUE       0x01
 #define CAL_SCALNAME           0x02
@@ -523,6 +524,15 @@ static const WCHAR LOCALE_NAME_SYSTEM_DEFAULT[] = {'!','s','y','s','-','d','e','
 #define CAL_SABBREVMONTHNAME13 0x2e
 #define CAL_SYEARMONTH         0x2f
 #define CAL_ITWODIGITYEARMAX   0x30
+#define CAL_SSHORTESTDAYNAME1  0x31
+#define CAL_SSHORTESTDAYNAME2  0x32
+#define CAL_SSHORTESTDAYNAME3  0x33
+#define CAL_SSHORTESTDAYNAME4  0x34
+#define CAL_SSHORTESTDAYNAME5  0x35
+#define CAL_SSHORTESTDAYNAME6  0x36
+#define CAL_SSHORTESTDAYNAME7  0x37
+#define CAL_SMONTHDAY          0x38
+#define CAL_SABBREVERASTRING   0x39
 
 /* Calendar types */
 #define CAL_GREGORIAN              1
